@@ -14,12 +14,13 @@
 
 module Crypt.SHA256 ( sha256sum ) where
 
-import Foreign
+import Foreign hiding ( unsafePerformIO )
 import Foreign.C.Types
 import Numeric (showHex)
 import Foreign.C.String ( withCString )
 import Data.ByteString.Unsafe (unsafeUseAsCStringLen)
 import qualified Data.ByteString as B
+import System.IO.Unsafe ( unsafePerformIO )
 
 sha256sum :: B.ByteString -> String
 sha256sum p = unsafePerformIO $
