@@ -157,7 +157,8 @@ copyRepoAndGoToChosenVersion opts repodir rfsource = do
   copyRepo
   withRepository opts ((RepoJob $ \repository -> goToChosenVersion repository opts) :: RepoJob ())
   putInfo opts $ text "Finished getting."
-      where copyRepo =
+      where copyRepo :: IO ()
+            copyRepo =
                 withRepository opts $ RepoJob $ \repository ->
                   if formatHas HashedInventory rfsource
                    then do
