@@ -17,6 +17,8 @@ module Darcs.External (
     sendmailPath, diffProgram, darcsProgram
   ) where
 
+import Prelude hiding ( catch )
+
 import qualified Ratified
 import Data.Maybe ( isJust, isNothing, maybeToList )
 import Control.Monad ( when, zipWithM_, filterM, liftM2 )
@@ -34,7 +36,8 @@ import System.Directory ( createDirectory, getDirectoryContents,
                           findExecutable )
 import System.Process ( runProcess, runInteractiveProcess, waitForProcess )
 import Control.Concurrent ( forkIO, newEmptyMVar, putMVar, takeMVar )
-import Control.Exception.Extensible ( bracket, try, finally, SomeException )
+import Control.Exception.Extensible
+    ( bracket, try, finally, catch, IOException, SomeException )
 import Data.Char ( toUpper )
 #if defined (HAVE_MAPI)
 import Foreign.C ( CString, withCString )
@@ -102,7 +105,7 @@ diffProgram = do
 -- |Get the name of the darcs executable (as supplied by @getProgName@)
 darcsProgram :: IO String
 darcsProgram = getProgName
--- Another option: getEnv "DARCS" `catch` \_ -> getProgName
+-- Another option: getEnv "DARCS" `catch` \(_ :: IOException) -> getProgName
 
 backupByRenaming :: FilePath -> IO ()
 backupByRenaming = backupBy rename
@@ -200,7 +203,7 @@ cloneTreeExcept except source dest =
             mk_dest   fp = dest   </> fp
         zipWithM_ cloneSubTree (map mk_source fps') (map mk_dest fps')
      else fail ("cloneTreeExcept: Bad source " ++ source)
-   `catch` fail ("cloneTreeExcept: Bad source " ++ source)
+   `catch` (\(_ :: IOException) -> fail ("cloneTreeExcept: Bad source " ++ source))
 
 cloneSubTree :: FilePath -> FilePath -> IO ()
 cloneSubTree source dest =
@@ -226,7 +229,7 @@ maybeURLCmd :: String -> String -> IO(Maybe(String))
 maybeURLCmd what url =
   do let prot = map toUpper $ takeWhile (/= ':') url
      fmap Just (getEnv ("DARCS_" ++ what ++ "_" ++ prot))
-             `catch` \_ -> return Nothing
+             `catch` \(_ :: IOException) -> return Nothing
 
 speculateRemote :: String -> FilePath -> IO () -- speculations are always Cachable
 #if defined(HAVE_CURL) || defined(HAVE_HTTP)
@@ -318,7 +321,7 @@ generateEmail h f t s cc body = do
         newline = B.singleton 10
 
 haveSendmail :: IO Bool
-haveSendmail = (sendmailPath >> return True) `catch` (\_ -> return False)
+haveSendmail = (sendmailPath >> return True) `catch` (\(_ :: IOException) -> return False)
 
 -- | Send an email, optionally containing a patch bundle
 --   (more precisely, its description and the bundle itself)
