@@ -61,8 +61,12 @@ import Control.Applicative ( (<$>), (<*>) )
 import Control.Monad ( when )
 import Control.Concurrent.MVar
 import Control.Exception.Extensible ( bracket_, catch, catchJust, SomeException
-                                    , block, unblock
                                     )
+#if (MIN_VERSION_base(4,4,0))
+import Control.Exception.Extensible ( mask )
+#else
+import Control.Exception.Extensible ( block, unblock )
+#endif
 import Data.IORef ( IORef, newIORef, readIORef, writeIORef )
 import Data.IORef ( modifyIORef )
 import Data.List ( isPrefixOf )
@@ -75,6 +79,9 @@ import System.Time ( calendarTimeToString, toCalendarTime, getClockTime )
 import System.Environment ( getEnv )
 import Prelude hiding (catch)
 
+#if !(MIN_VERSION_base(4,4,0))
+mask f = block (f unblock)
+#endif
 
 windows :: Bool
 windows = "mingw" `isPrefixOf` os
@@ -106,12 +113,12 @@ withAtexit prog =
         exit
         prog
   where
-    exit = block $ do
+    exit = mask $ \unmask -> do
         Just actions <- swapMVar atexitActions Nothing
         -- from now on atexit will not register new actions
-        mapM_ runAction actions
-    runAction action =
-        catch (unblock action) $ \(exn :: SomeException) -> do
+        mapM_ (runAction unmask) actions
+    runAction unmask action =
+        catch (unmask action) $ \(exn :: SomeException) -> do
             hPutStrLn stderr $ "Exception thrown by an atexit registered action:"
             hPutStrLn stderr $ show exn
 
@@ -239,7 +246,7 @@ resetCRCWarnings = writeIORef _crcWarningList []
 
 _badSourcesList :: IORef [String]
 _badSourcesList = unsafePerformIO $ newIORef []
-{- NOINLINE _badSourcesList -}
+{-# NOINLINE _badSourcesList #-}
 
 
 addBadSource :: String -> IO ()
@@ -258,7 +265,7 @@ isBadSource = do
 
 _reachableSourcesList :: IORef [String]
 _reachableSourcesList = unsafePerformIO $ newIORef []
-{- NOINLINE _reachableSourcesList -}
+{-# NOINLINE _reachableSourcesList #-}
 
 
 addReachableSource :: String -> IO ()
