@@ -16,25 +16,31 @@
 -- Boston, MA 02110-1301, USA.
 
 module Darcs.Patch.Info ( PatchInfo(..), patchinfo, invertName,
-                          idpatchinfo, addJunk,
+                          idpatchinfo, addJunk, makePatchname,
                           makeFilename, makeAltFilename, readPatchInfo,
                           justName, justAuthor, justLog, repopatchinfo,
                           RepoPatchInfo, humanFriendly, toXml, piDate,
                           setPiDate, piDateString, piDateBytestring,
                           piName, piRename, piAuthor, piTag, piLog,
-                          showPatchInfo, isTag
+                          showPatchInfo, isTag, readPatchInfos, escapeXML
                         ) where
-import Text.Html hiding (name, text)
+import Text.Html hiding (name, text, option)
 import System.Random ( randomRIO )
 import Numeric ( showHex )
-import Control.Monad ( when )
+import Control.Monad ( when, unless )
 
-import ByteStringUtils
-import qualified Data.ByteString       as B  (length, splitAt, null, drop
+import ByteStringUtils ( unlinesPS, packStringToUTF8, unpackPSFromUTF8, decodeLocale)
+import qualified Darcs.Patch.ReadMonads as RM ( take )
+import Darcs.Patch.ReadMonads as RM ( skipSpace, char,
+                                      takeTill, anyChar, ParserM,
+                                      option, parseStrictly,
+                                      takeTillChar,
+                                      linesStartingWithEndingWith)
+import qualified Data.ByteString       as B  (length, splitAt, null
                                              ,isPrefixOf, tail, concat
                                              ,empty, head, cons, append
                                              ,ByteString )
-import qualified Data.ByteString.Char8 as BC (index, head, unpack, pack, break)
+import qualified Data.ByteString.Char8 as BC (index, head, unpack, pack)
 import Data.List( isPrefixOf )
 
 import Printer ( renderString, Doc, packedString,
@@ -86,8 +92,8 @@ addJunk pinf =
     do x <- randomRIO (0,2^(128 ::Integer) :: Integer)
        when (_piLog pinf /= ignoreJunk (_piLog pinf)) $
             do putStrLn "Lines beginning with 'Ignore-this: ' will be ignored."
-               yorn <- promptYorn "Proceed? "
-               when (yorn == 'n') $ fail "User cancelled because of Ignore-this."
+               confirmed <- promptYorn "Proceed? "
+               unless confirmed $ fail "User cancelled because of Ignore-this."
        return $ pinf { _piLog = BC.pack (head ignored++showHex x ""):
                                  _piLog pinf }
 
@@ -200,7 +206,7 @@ toXml pi =
     <+> text "date='" <> escapeXMLByteString (_piDate pi) <> text "'"
     <+> text "local_date='" <> escapeXML (friendlyD $ _piDate pi) <> text "'"
     <+> text "inverted='" <> text (show $ isInverted pi) <> text "'"
-    <+> text "hash='" <> text (makeFilename pi) <> text "'>"
+    <+> text "hash='" <> text (makePatchname pi) <> text "'>"
  $$     prefix "\t" (
             text "<name>" <> escapeXMLByteString (_piName pi) <> text "</name>"
          $$ commentsAsXml (_piLog pi))
@@ -249,11 +255,23 @@ makeAltFilename pi@(PatchInfo { isInverted = False }) =
 makeAltFilename pi@(PatchInfo { isInverted = True}) =
     makeAltFilename (pi { isInverted = False }) ++ "-inverted"
 
--- This makes darcs-1 (non-hashed repos) filenames, and is also generally used in both in
--- hashed and non-hashed repo code for making patch "hashes"
+-- | This makes darcs-1 (non-hashed repos) filenames, and is also generally
+-- used in both in hashed and non-hashed repo code for making patch "hashes".
+--
+-- The name consists of three segments:
+--
+--  * timestamp (ISO8601-compatible yyyymmmddHHMMSS, UTC)
+--
+--  * SHA1 hash of the author
+--
+--  * SHA1 hash of the patch name, author, date, log, and \"inverted\"
+--    flag.
 makeFilename :: PatchInfo -> String
-makeFilename pi =
-    showIsoDateTime d++"-"++sha1_a++"-"++sha1PS sha1_me++".gz"
+makeFilename pi = makePatchname pi ++ ".gz"
+
+makePatchname :: PatchInfo -> String
+makePatchname pi =
+    showIsoDateTime d++"-"++sha1_a++"-"++sha1PS sha1_me
         where b2ps True = BC.pack "t"
               b2ps False = BC.pack "f"
               sha1_me = B.concat [_piName pi,
@@ -315,6 +333,7 @@ instance  Show PatchInfo  where
 -- > ]
 --
 -- note that below I assume the name has no newline in it.
+-- See 'readPatchInfo' for the inverse operation.
 showPatchInfo :: PatchInfo -> Doc
 showPatchInfo pi =
     blueText "[" <> packedString (_piName pi)
@@ -326,49 +345,39 @@ showPatchInfo pi =
               where mul [] = text "\n"
                     mul (s:ss) = text "\n " <> packedString s <> mul ss
 
+-- |Parser for 'PatchInfo' as stored in patch bundles and inventory files,
+-- for example:
 --
--- Note, Data.ByteString rewrites break ((==) x) into the memchr-based
--- breakByte. For this rule to fire, we keep it in prefix application form
+-- > [Document the foo interface
+-- > John Doe <john.doe@example.com>**20110615084241
+-- >  Ignore-this: 85b94f67d377c4ab671101266ef9c229
+-- >  Nobody knows what a 'foo' is, so describe it.
+-- > ]
 --
+-- See 'showPatchInfo' for the inverse operation.
+readPatchInfo :: ParserM m => m (PatchInfo)
+readPatchInfo = do
+  skipSpace
+  char '['
+  name <- takeTillChar '\n'
+  _ <- anyChar
+  author <- takeTillChar '*'
+  s2 <- RM.take 2
+  ct <- takeTill (\c->c==']'||c=='\n')
+  option () (char '\n' >> return ()) -- consume newline char, if present
+  log <- linesStartingWithEndingWith ' ' ']'
+  return $ PatchInfo { _piDate = ct
+                     , _piName = name
+                     , _piAuthor = author
+                     , _piLog = log
+                     , isInverted = BC.index s2 1 /= '*'
+                     }
 
-readPatchInfo :: B.ByteString -> Maybe (PatchInfo, B.ByteString)
-readPatchInfo s | B.null (dropSpace s) = Nothing
-readPatchInfo s =
-    if BC.head (dropSpace s) /= '[' -- ]
-    then Nothing
-    else case BC.break ((==) '\n') $ B.tail $ dropSpace s of
-         (name,s') ->
-             case BC.break ((==) '*') $ B.tail s' of
-             (author,s2) ->
-                 case BC.break (\c->c==']'||c=='\n') $ B.drop 2 s2 of
-                 (ct,s''') ->
-                     do (log, s4) <- linesStartingWithEndingWith ' ' ']' $ dn s'''
-                        return $ (PatchInfo { _piDate = ct
-                                            , _piName = name
-                                            , _piAuthor = author
-                                            , _piLog = log
-                                            , isInverted = BC.index s2 1 /= '*'
-                                            }, s4)
-    where dn x = if B.null x || BC.head x /= '\n' then x else B.tail x
-
-linesStartingWithEndingWith :: Char -> Char -> B.ByteString
-                                -> Maybe ([B.ByteString],B.ByteString)
-linesStartingWithEndingWith st en s = lswew s
-    where
-  lswew x | B.null x = Nothing
-  lswew x =
-    if BC.head x == en
-    then Just ([], B.tail x)
-    else if BC.head x /= st
-         then Nothing
-         else case BC.break ((==) '\n') $ B.tail x of
-              (l,r) -> case lswew $ B.tail r of
-                       Just (ls,r') -> Just (l:ls,r')
-                       Nothing ->
-                           case breakLastPS en l of
-                           Just (l2,_) ->
-                               Just ([l2], B.drop (B.length l2+2) x)
-                           Nothing -> Nothing
+readPatchInfos :: B.ByteString -> [PatchInfo]
+readPatchInfos inv | B.null inv = []
+readPatchInfos inv = case parseStrictly readPatchInfo inv of
+                     Just (pinfo,r) -> pinfo : readPatchInfos r
+                     _ -> []
 
 htmlPatchInfo :: RepoPatchInfo -> Html
 htmlPatchInfo (RPI r pi) =
