@@ -14,7 +14,6 @@
 -- along with this program; if not, write to the Free Software Foundation,
 -- Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-{-# OPTIONS_GHC -cpp -fglasgow-exts #-}
 {-# LANGUAGE CPP, ScopedTypeVariables #-}
 
 #include "gadts.h"
@@ -22,22 +21,29 @@
 module Darcs.Repository.HashedRepo ( revertTentativeChanges, finalizeTentativeChanges,
                                      cleanPristine,
                                      copyPristine, copyPartialsPristine,
-                                     applyToTentativePristine,
+                                     applyToTentativePristine, addToSpecificInventory,
                                      addToTentativeInventory, removeFromTentativeInventory,
-                                     readRepo, readTentativeRepo, writeAndReadPatch,
+                                     readRepo, readTentativeRepo,
+                                     readRepoUsingSpecificInventory, writeAndReadPatch,
                                      writeTentativeInventory, copyRepo,
-                                     readHashedPristineRoot, pris2inv, copySources
+                                     readHashedPristineRoot, pris2inv, copySources,
+                                     listInventories, writePatchIfNecessary,
+                                     readRepoFromInventoryList, readPatchIds
                                    ) where
 
+import Prelude hiding ( catch )
+
 import System.Directory ( createDirectoryIfMissing )
 import System.FilePath.Posix( (</>) )
 import System.IO.Unsafe ( unsafeInterleaveIO )
 import System.IO ( stderr, hPutStrLn )
-import Data.List ( delete, filter )
+import Data.List ( delete )
 import Control.Monad ( unless )
+import Control.Applicative ( (<$>) )
+import Control.Exception ( catch, IOException )
 
 import Workaround ( renameFile )
-import Darcs.Flags ( DarcsFlag, Compression )
+import Darcs.Flags ( Compression, RemoteDarcs )
 import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet )
 #ifdef GADT_WITNESSES
 import Darcs.Patch.Set ( Origin )
@@ -46,19 +52,21 @@ import Darcs.RepoPath ( FilePathLike, ioAbsoluteOrRemote, toPath )
 import Darcs.Repository.Cache ( Cache(..), CacheLoc(..), fetchFileUsingCache,
                                 speculateFilesUsingCache, writeFileUsingCache,
                                 unionCaches, repo2cache, okayHash, takeHash,
-                                HashedDir(..),WritableOrNot(..), hashedDir,
+                                HashedDir(..), hashedDir,
                                 peekInCache )
 import qualified Darcs.Repository.Cache as DarcsCache
 import Darcs.Repository.HashedIO ( copyHashed, copyPartialsHashed,
                                    cleanHashdir )
 import Darcs.Repository.InternalTypes ( Repository(..), extractCache, modifyCache )
-import Darcs.Repository.Prefs ( globalCacheDir )
-import Darcs.Hopefully ( PatchInfoAnd, patchInfoAndPatch, info,
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, patchInfoAndPatch, info,
                          extractHash, createHashed )
 import Darcs.Patch ( RepoPatch, Patchy, showPatch, readPatch, apply )
+import Darcs.Patch.Apply ( Apply, ApplyState )
+import Darcs.Patch.ReadMonads ( parseStrictly )
 import Darcs.Patch.Depends ( commuteToEnd, slightlyOptimizePatchset )
 import Darcs.Patch.Info ( PatchInfo, showPatchInfo, humanFriendly, readPatchInfo )
-import Darcs.Witnesses.Ordered ( unsafeCoerceP, reverseRL, reverseFL, (+<+) )
+import Darcs.Witnesses.Unsafe ( unsafeCoerceP )
+import Darcs.Witnesses.Ordered ( reverseRL, reverseFL, (+<+) )
 
 import ByteStringUtils ( gzReadFilePS, dropSpace )
 import qualified Data.ByteString as B (null, length, empty
@@ -66,32 +74,35 @@ import qualified Data.ByteString as B (null, length, empty
 import qualified Data.ByteString.Char8 as BC (unpack, dropWhile, break, pack)
 
 import Printer ( Doc, hcat, (<>), ($$), renderString, renderPS, text, invisiblePS )
+import Darcs.ColorPrinter () -- for instance Show Doc
 import Crypt.SHA256 ( sha256sum )
-import Darcs.External ( copyFileOrUrl, cloneFile, fetchFilePS, Cachable( Uncachable ) )
+import Darcs.External ( copyFileOrUrl, cloneFile, fetchFilePS, gzFetchFilePS,
+    Cachable( Uncachable ) )
 import Darcs.Lock ( writeBinFile, writeDocBinFile, writeAtomicFilePS, appendBinFile, appendDocBinFile )
 import Darcs.Utils ( withCurrentDirectory )
 import Progress ( beginTedious, endTedious, debugMessage, finishedOneIO )
 #include "impossible.h"
-import Darcs.Witnesses.Ordered ( FL(..), RL(..), (:>>)(..), mapRL, mapFL )
+import Darcs.Witnesses.Ordered ( FL(..), RL(..), (:>)(..), mapRL, mapFL )
 import Darcs.Witnesses.Sealed ( Sealed(..), seal, unseal, mapSeal )
 import Darcs.Global ( darcsdir )
 
 import Storage.Hashed.Darcs( hashedTreeIO, readDarcsHashedNosize, readDarcsHashed,
-                             writeDarcsHashed, readDarcsHashedDir, darcsLocation,
+                             writeDarcsHashed,
                              decodeDarcsHash, decodeDarcsSize )
-import Storage.Hashed.Tree( treeHash, ItemType(..) )
+import Storage.Hashed.Tree( treeHash, Tree )
 import Storage.Hashed.Hash( encodeBase16, Hash(..) )
 
-
-applyHashed' fs root p = do case root of
+applyHashed'
+   :: (Apply p, ApplyState p ~ Tree) => Hash -> p C(x y) -> IO String
+applyHashed' root p = do case root of
                               (SHA256 _) -> return ()
                               _ -> fail $ "Cannot handle hash: " ++ show root
-                            s <- readDarcsHashedNosize "_darcs/pristine.hashed" root
-                            (_, t) <- (hashedTreeIO (apply fs p) s "_darcs/pristine.hashed")
-                            return $ BC.unpack . encodeBase16 $ treeHash t
+                         s <- readDarcsHashedNosize "_darcs/pristine.hashed" root
+                         (_, t) <- (hashedTreeIO (apply p) s "_darcs/pristine.hashed")
+                         return $ BC.unpack . encodeBase16 $ treeHash t
 
-applyHashed :: Patchy q => [DarcsFlag] -> String -> q C(x y) -> IO String
-applyHashed fs h p = applyHashed' fs hash p `catch` \_ -> do
+applyHashed :: (ApplyState q ~ Tree, Patchy q) => String -> q C(x y) -> IO String
+applyHashed h p = applyHashed' hash p `catch` \(_ :: IOException) -> do
                           hPutStrLn stderr warn
                           inv <- gzReadFilePS invpath
                           let oldroot = BC.pack $ inv2pris inv
@@ -103,7 +114,7 @@ applyHashed fs h p = applyHashed' fs hash p `catch` \_ -> do
                           writeDocBinFile invpath $ pris2inv newroot inv
                           cleanHashdir (Ca []) HashedPristineDir [newroot]
                           hPutStrLn stderr "Pristine conversion done..."
-                          applyHashed' fs root p
+                          applyHashed' root p
   where invpath = darcsdir ++ "/hashed_inventory"
         hash = decodeDarcsHash $ BC.pack h
         warn = "WARNING: Doing a one-time conversion of pristine format.\n" ++
@@ -115,7 +126,7 @@ revertTentativeChanges =
        i <- gzReadFilePS (darcsdir++"/hashed_inventory")
        writeBinFile (darcsdir++"/tentative_pristine") $ "pristine:" ++ inv2pris i
 
-finalizeTentativeChanges :: RepoPatch p => Repository p C(r u t) -> Compression -> IO ()
+finalizeTentativeChanges :: (RepoPatch p, ApplyState p ~ Tree) => Repository p C(r u t) -> Compression -> IO ()
 finalizeTentativeChanges r compr =
     do let t = darcsdir++"/tentative_hashed_inventory"
        -- first let's optimize it...
@@ -134,7 +145,7 @@ finalizeTentativeChanges r compr =
 readHashedPristineRoot :: Repository p C(r u t) -> IO (Maybe String)
 readHashedPristineRoot (Repo d _ _ _) =
     withCurrentDirectory d $ do
-      i <- (Just `fmap` gzReadFilePS (darcsdir++"/hashed_inventory")) `catch` (\_ -> return Nothing)
+      i <- (Just `fmap` gzReadFilePS (darcsdir++"/hashed_inventory")) `catch` (\(_ :: IOException) -> return Nothing)
       return $ inv2pris `fmap` i
 
 cleanPristine :: Repository p C(r u t) -> IO ()
@@ -144,15 +155,20 @@ cleanPristine r@(Repo d _ _ _) = withCurrentDirectory d $
       i <- gzReadFilePS (darcsdir++"/hashed_inventory")
       cleanHashdir (extractCache r) HashedPristineDir [inv2pris i]
 
+addToSpecificInventory :: RepoPatch p => String -> Cache -> Compression
+                           -> PatchInfoAnd p C(x y) -> IO FilePath
+addToSpecificInventory invPath c compr p = do
+  let invFile = darcsdir </> invPath
+  hash <- snd `fmap` writePatchIfNecessary c compr p
+  appendDocBinFile invFile $ showPatchInfo $ info p
+  appendBinFile invFile $ "\nhash: " ++ hash ++ "\n"
+  return $ darcsdir </> "patches" </> hash
+
 addToTentativeInventory :: RepoPatch p => Cache -> Compression
                            -> PatchInfoAnd p C(x y) -> IO FilePath
-addToTentativeInventory c compr p =
-    do hash <- snd `fmap` writePatchIfNecesary c compr p
-       appendDocBinFile (darcsdir++"/tentative_hashed_inventory") $ showPatchInfo $ info p
-       appendBinFile (darcsdir++"/tentative_hashed_inventory") $ "\nhash: " ++ hash ++ "\n"
-       return $ darcsdir++"/patches/" ++ hash
+addToTentativeInventory = addToSpecificInventory "tentative_hashed_inventory"
 
-removeFromTentativeInventory :: RepoPatch p => Repository p C(r u t) -> Compression
+removeFromTentativeInventory :: (RepoPatch p, ApplyState p ~ Tree) => Repository p C(r u t) -> Compression
                                 -> FL (PatchInfoAnd p) C(x t) -> IO ()
 removeFromTentativeInventory repo compr to_remove =
        -- FIXME: This algorithm should be *far* simpler.  All we need do is
@@ -160,13 +176,13 @@ removeFromTentativeInventory repo compr to_remove =
        -- patchset.  The commutation behavior of PatchInfoAnd should track
        -- which patches need to be rewritten for us.
     do allpatches <- readTentativeRepo repo "."
-       _ :>> skipped <- return $ commuteToEnd (reverseFL to_remove) allpatches
+       _ :> skipped <- return $ commuteToEnd (reverseFL to_remove) allpatches
        okay <- simpleRemoveFromTentativeInventory repo compr
                (mapFL info to_remove ++ mapRL info skipped)
        unless okay $ bug "bug in HashedRepo.removeFromTentativeInventory"
        sequence_ $ mapFL (addToTentativeInventory (extractCache repo) compr) (reverseRL skipped)
 
-simpleRemoveFromTentativeInventory :: forall p C(r u t). RepoPatch p =>
+simpleRemoveFromTentativeInventory :: forall p C(r u t). (RepoPatch p, ApplyState p ~ Tree) =>
                                           Repository p C(r u t) -> Compression -> [PatchInfo] -> IO Bool
 simpleRemoveFromTentativeInventory repo compr pis = do
     inv <- readTentativeRepo repo "."
@@ -186,34 +202,38 @@ writeHashFile :: Cache -> Compression -> HashedDir -> Doc -> IO String
 writeHashFile c compr subdir d = do debugMessage $ "Writing hash file to "++(hashedDir subdir)
                                     writeFileUsingCache c compr subdir $ renderPS d
 
-readRepo :: RepoPatch p => Repository p C(r u t) -> String -> IO (PatchSet p C(Origin r))
-readRepo repo d = do
-  realdir <- toPath `fmap` ioAbsoluteOrRemote d
-  Sealed ps <- readRepoPrivate (extractCache repo) realdir "hashed_inventory" `catch`
+readRepo :: (RepoPatch p, ApplyState p ~ Tree) => Repository p C(r u t) -> String -> IO (PatchSet p C(Origin r))
+readRepo repo d = readRepoUsingSpecificInventory "hashed_inventory" repo d
+
+readTentativeRepo :: (RepoPatch p, ApplyState p ~ Tree) => Repository p C(r u t) -> String -> IO (PatchSet p C(Origin t))
+readTentativeRepo repo d =
+  readRepoUsingSpecificInventory "tentative_hashed_inventory" repo d
+
+readRepoUsingSpecificInventory :: (RepoPatch p, ApplyState p ~ Tree) => String -> Repository p C(r u t)
+  -> String -> IO (PatchSet p C(Origin s))
+readRepoUsingSpecificInventory invPath repo dir = do
+  realdir <- toPath `fmap` ioAbsoluteOrRemote dir
+  Sealed ps <- readRepoPrivate (extractCache repo) realdir invPath `catch`
                  (\e -> do hPutStrLn stderr ("Invalid repository:  " ++ realdir)
                            ioError e)
   return $ unsafeCoerceP ps
 
-readTentativeRepo :: RepoPatch p => Repository p C(r u t) -> String -> IO (PatchSet p C(Origin t))
-readTentativeRepo repo d = do
-  realdir <- toPath `fmap` ioAbsoluteOrRemote d
-  Sealed ps <- readRepoPrivate (extractCache repo) realdir "tentative_hashed_inventory" `catch`
-                 (\e -> do hPutStrLn stderr ("Invalid repository:  " ++ realdir)
-                           ioError e)
-  return $ unsafeCoerceP ps
+readRepoPrivate :: (RepoPatch p, ApplyState p ~ Tree) => Cache -> FilePath -> FilePath -> IO (SealedPatchSet p C(Origin))
+readRepoPrivate cache d iname = do
+  inventory <- readInventoryPrivate cache (d </> "_darcs") iname
+  readRepoFromInventoryList cache inventory
 
-readRepoPrivate :: RepoPatch p => Cache -> FilePath -> FilePath -> IO (SealedPatchSet p C(Origin))
-readRepoPrivate cache d iname =
- do inventory <- readInventoryPrivate cache (d </> "_darcs") iname
-    parseinvs inventory
-    where read_patches :: RepoPatch p => [(PatchInfo, String)]
+readRepoFromInventoryList :: (RepoPatch p, ApplyState p ~ Tree) => Cache
+  -> (Maybe String, [(PatchInfo, String)]) -> IO (SealedPatchSet p C(Origin))
+readRepoFromInventoryList cache inventory = parseinvs inventory
+    where read_patches :: (RepoPatch p, ApplyState p ~ Tree) => [(PatchInfo, String)]
                        -> IO (Sealed (RL (PatchInfoAnd p) C(x)))
           read_patches [] = return $ seal NilRL
           read_patches allis@((i1,h1):is1) =
               lift2Sealed (\p rest -> i1 `patchInfoAndPatch` p :<: rest)
                           (rp is1)
                           (createHashed h1 (const $ speculate h1 allis >> parse i1 h1))
-              where rp :: RepoPatch p => [(PatchInfo, String)]
+              where rp :: (RepoPatch p, ApplyState p ~ Tree) => [(PatchInfo, String)]
                        -> IO (Sealed (RL (PatchInfoAnd p) C(x)))
                     rp [] = return $ seal NilRL
                     rp [(i,h),(il,hl)] =
@@ -223,7 +243,7 @@ readRepoPrivate cache d iname =
                     rp ((i,h):is) = lift2Sealed (\p rest -> i `patchInfoAndPatch` p :<: rest)
                                                 (rp is)
                                                 (createHashed h (parse i))
-          read_tag :: RepoPatch p => (PatchInfo, String) -> IO (Sealed (PatchInfoAnd p C(x)))
+          read_tag :: (RepoPatch p, ApplyState p ~ Tree) => (PatchInfo, String) -> IO (Sealed (PatchInfoAnd p C(x)))
           read_tag (i,h) = mapSeal (patchInfoAndPatch i) `fmap` createHashed h (parse i)
           speculate :: String -> [(PatchInfo, String)] -> IO ()
           speculate h is =
@@ -234,11 +254,11 @@ readRepoPrivate cache d iname =
           parse i h = do debugMessage ("Reading patch file: "++ show (humanFriendly i))
                          (fn,ps) <- fetchFileUsingCache cache HashedPatchesDir h
                          case readPatch ps of
-                           Just (p,_) -> return p
+                           Just p -> return p
                            Nothing -> fail $ unlines ["Couldn't parse file "++fn,
                                                       "which is patch",
                                                       renderString $ humanFriendly i]
-          parseinvs :: RepoPatch p => (Maybe String, [(PatchInfo, String)])
+          parseinvs :: (RepoPatch p, ApplyState p ~ Tree) => (Maybe String, [(PatchInfo, String)])
                     -> IO (SealedPatchSet p C(Origin))
           parseinvs (Nothing, ris) = mapSeal (\ps -> PatchSet ps NilRL)
                                      `fmap` (read_patches $ reverse ris)
@@ -249,7 +269,7 @@ readRepoPrivate cache d iname =
                                              unsafeInterleaveIO (read_patches $
                                                                  reverse ris)
                                          return $ seal $ PatchSet ps ts
-          read_ts :: RepoPatch p => (PatchInfo, String) -> String -> IO (Sealed (RL (Tagged p) C(Origin)))
+          read_ts :: (RepoPatch p, ApplyState p ~ Tree) => (PatchInfo, String) -> String -> IO (Sealed (RL (Tagged p) C(Origin)))
           read_ts tag0 h0 =
               do contents <- unsafeInterleaveIO $ readTaggedInventory cache h0
                  let is = reverse $ case contents of (Just _, _:ris0) -> ris0
@@ -285,32 +305,36 @@ readTaggedInventory cache ihash = do
                   _ -> return (Nothing,i)
     return (rest, readPatchIds str)
 
-copyRepo :: RepoPatch p => Repository p C(r u t) -> [DarcsFlag] -> String -> IO ()
-copyRepo repo@(Repo outr _ _ _) opts inr = do
+copyRepo :: RepoPatch p => Repository p C(r u t) -> RemoteDarcs -> String -> IO ()
+copyRepo repo@(Repo outr _ _ _) remote inr = do
     createDirectoryIfMissing False (outr++"/"++darcsdir++"/inventories")
-    copyFileOrUrl opts (inr++"/"++darcsdir++"/hashed_inventory") (outr++"/"++darcsdir++"/hashed_inventory")
+    copyFileOrUrl remote (inr </> darcsdir </> "hashed_inventory")
+                         (outr </> darcsdir </> "hashed_inventory")
                   Uncachable -- no need to copy anything but hashed_inventory!
     copySources repo inr
     debugMessage "Done copying hashed inventory."
 
+-- |'copySources' copies the prefs/sources file to the local repo, from the
+-- remote, having first filtered the local filesystem sources.
 copySources :: RepoPatch p => Repository p C(r u t) -> String -> IO ()
 copySources repo@(Repo outr _ _ _) inr = do
-    let repoCache = extractCache $ modifyCache repo dropGlobalCaches
+    let repoCache = extractCache $ modifyCache repo dropNonRepos
     appendBinFile (outr++"/"++darcsdir++"/prefs/sources") (show $ repo2cache inr `unionCaches` repoCache )
   where
-    dropGlobalCaches (Ca cache) = Ca $ filter notGlobalCache cache
-    notGlobalCache xs = case xs of
-                         Cache DarcsCache.Directory _ _ -> False
-                         _                              -> True
+    dropNonRepos (Ca cache) = Ca $ filter notRepo cache
+    notRepo xs = case xs of
+        Cache DarcsCache.Directory _ _ -> False
+        Cache _ DarcsCache.Writable _  -> False
+        _                              -> True
 
 writeAndReadPatch :: RepoPatch p => Cache -> Compression -> PatchInfoAnd p C(x y)
                      -> IO (PatchInfoAnd p C(x y))
-writeAndReadPatch c compr p =    do (i,h) <- writePatchIfNecesary c compr p
+writeAndReadPatch c compr p =    do (i,h) <- writePatchIfNecessary c compr p
                                     unsafeInterleaveIO $ readp h i
     where parse i h = do debugMessage ("Rereading patch file: "++ show (humanFriendly i))
                          (fn,ps) <- fetchFileUsingCache c HashedPatchesDir h
                          case readPatch ps of
-                           Just (x,_) -> return x
+                           Just x -> return x
                            Nothing -> fail $ unlines ["Couldn't parse patch file "++fn,
                                                       "which is",
                                                       renderString $ humanFriendly i]
@@ -338,14 +362,14 @@ writeInventoryPrivate :: RepoPatch p => Cache -> String -> Compression
                         -> PatchSet p C(Origin x) -> IO (Maybe String)
 writeInventoryPrivate _ _ _ (PatchSet NilRL NilRL) = return Nothing
 writeInventoryPrivate cache _ compr (PatchSet x NilRL) =
-  do inventory <- sequence $ mapRL (writePatchIfNecesary cache compr) x
+  do inventory <- sequence $ mapRL (writePatchIfNecessary cache compr) x
      let inventorylist = hcat (map pihash $ reverse inventory)
      hash <- writeHashFile cache compr HashedInventoriesDir inventorylist
      return $ Just hash
 writeInventoryPrivate cache k compr (PatchSet x xs@(Tagged t _ _ :<: _)) =
   do resthash <- write_ts xs
      finishedOneIO k $ maybe "" id resthash
-     inventory <- sequence $ mapRL (writePatchIfNecesary cache compr) (x+<+t:<:NilRL)
+     inventory <- sequence $ mapRL (writePatchIfNecessary cache compr) (x+<+t:<:NilRL)
      let inventorylist = hcat (map pihash $ reverse inventory)
          inventorycontents =
              case resthash of
@@ -359,9 +383,9 @@ writeInventoryPrivate cache k compr (PatchSet x xs@(Tagged t _ _ :<: _)) =
               writeInventoryPrivate cache k compr $ PatchSet pps tts
           write_ts NilRL = return Nothing
 
-writePatchIfNecesary :: RepoPatch p => Cache -> Compression
+writePatchIfNecessary :: RepoPatch p => Cache -> Compression
                         -> PatchInfoAnd p C(x y) -> IO (PatchInfo, String)
-writePatchIfNecesary c compr hp =
+writePatchIfNecessary c compr hp =
     seq infohp $ case extractHash hp of
                    Right h -> return (infohp, h)
                    Left p -> (\h -> (infohp, h)) `fmap`
@@ -373,7 +397,7 @@ pihash (pinf,hash) = showPatchInfo pinf $$ text ("hash: " ++ hash ++ "\n")
 
 readInventoryPrivate :: Cache -> String -> String -> IO (Maybe String, [(PatchInfo, String)])
 readInventoryPrivate _ d iname = do
-    i <- skipPristine `fmap` fetchFilePS (d </> iname) Uncachable
+    i <- skipPristine `fmap` gzFetchFilePS (d </> iname) Uncachable
     (rest,str) <- case BC.break ((==)'\n') i of
                   (swt,pistr) | swt == BC.pack "Starting with inventory:" ->
                     case BC.break ((==)'\n') $ B.tail pistr of
@@ -383,24 +407,22 @@ readInventoryPrivate _ d iname = do
                   _ -> return (Nothing, i)
     return (rest, readPatchIds str)
 
-readInventories :: Cache -> String -> IO [[(PatchInfo, String)]]
-readInventories cache ihash = do
-    (fn,i_and_p) <- fetchFileUsingCache cache HashedInventoriesDir ihash
-    let i = skipPristine i_and_p
-    (rest,str) <- case BC.break ((==)'\n') i of
-                  (swt,pistr) | swt == BC.pack "Starting with inventory:" ->
-                    case BC.break ((==)'\n') $ B.tail pistr of
-                    (h,thisinv) | okayHash $ BC.unpack h ->
-                      do r <- unsafeInterleaveIO $
-                              readInventories cache (BC.unpack h) -- again. no.
-                         return (r,thisinv)
-                    _ -> fail $ "Bad hash in file " ++ fn
-                  _ -> return ([],i)
-    return $ reverse (readPatchIds str) : rest
+listInventories :: IO [String]
+listInventories = do
+  x <- fst <$> readInventoryPrivate (Ca []) darcsdir "hashed_inventory"
+  maybe (return []) f x
+ where
+  f i = do
+    x <- fst <$> readInventoryPrivate (Ca []) (darcsdir </> "inventories") i
+    (i :) <$> maybe (return []) f x
 
+-- | 'readPatchIds inventory' parses the content of a hashed_inventory file
+-- after the "pristine:" and "Starting with inventory:" header lines have
+-- been removed.  The second value in the resulting tuples is the file hash
+-- of the associated patch (the "hash:" line).
 readPatchIds :: B.ByteString -> [(PatchInfo, String)]
 readPatchIds inv | B.null inv = []
-readPatchIds inv = case readPatchInfo inv of
+readPatchIds inv = case parseStrictly readPatchInfo inv of
                      Nothing -> []
                      Just (pinfo,r) ->
                          case readHash r of
@@ -415,14 +437,14 @@ readHash s = let s' = dropSpace s
                 then Nothing
                 else Just (BC.unpack $ B.tail h,r)
 
-applyPristine :: Patchy q => [DarcsFlag] -> String -> String -> q C(x y) -> IO ()
-applyPristine opts d iname p =
+applyPristine :: (ApplyState q ~ Tree, Patchy q) => String -> String -> q C(x y) -> IO ()
+applyPristine d iname p =
     do i <- gzReadFilePS (d++"/"++iname)
-       h <- applyHashed opts (inv2pris i) p
+       h <- applyHashed (inv2pris i) p
        writeDocBinFile (d++"/"++iname) $ pris2inv h i
 
-applyToTentativePristine :: Patchy q => [DarcsFlag] -> q C(x y) -> IO ()
-applyToTentativePristine opts p = applyPristine opts "." (darcsdir++"/tentative_pristine") p
+applyToTentativePristine :: (ApplyState q ~ Tree, Patchy q) => q C(x y) -> IO ()
+applyToTentativePristine p = applyPristine "." (darcsdir++"/tentative_pristine") p
 
 copyPristine :: Cache -> Compression -> String -> String -> IO ()
 copyPristine c compr d iname = do
