@@ -50,14 +50,13 @@ import Darcs.Commands ( CommandArgs( CommandOnly, SuperCommandOnly, SuperCommand
 import Darcs.Commands.GZCRCs ( doCRCWarnings )
 import Darcs.Global ( atexit )
 import Darcs.External ( viewDoc )
-import Darcs.Global ( setDebugMode, setSshControlMasterDisabled,
-                      setTimingsMode, setVerboseMode )
+import Darcs.Global ( setDebugMode, setTimingsMode )
 import Darcs.Match ( checkMatchSyntax )
 import Progress ( setProgressMode )
 import Darcs.RepoPath ( getCurrentDirectory )
 import Darcs.Test ( runPosthook, runPrehook )
 import Darcs.Utils ( formatPath )
-import Data.List ( intersperse )
+import Data.List ( intercalate )
 import Printer ( text )
 import URL ( setDebugHTTP, disableHTTPPipelining )
 
@@ -87,11 +86,12 @@ runCommand msuper cmd args = do
       | Help `elem` opts -> viewDoc $ text $ getCommandHelp msuper cmd
       | ListOptions `elem` opts  -> do
            setProgressMode False
-           commandPrereq cmd opts
+-- Warning:  A do-notation statement discarded a result of type Either String ().
+           _ <- commandPrereq cmd opts
            file_args <- commandGetArgPossibilities cmd
            putStrLn $ unlines $ getOptionsOptions (opts1++opts2) : file_args
       | otherwise -> considerRunning msuper cmd (addVerboseIfDebug opts) extra
-    (_,_,ermsgs) -> do fail $ chompNewline(unlines ermsgs)
+    (_,_,ermsgs) -> fail $ chompNewline(unlines ermsgs)
     where addVerboseIfDebug opts | DebugVerbose `elem` opts = Debug:Verbose:opts
                                  | otherwise = opts
 
@@ -109,21 +109,10 @@ considerRunning msuper cmd opts old_extra = do
     extra <- (commandArgdefaults cmd) specops cwd old_extra
     when (Disable `elem` specops) $
       fail $ "Command "++commandName cmd++" disabled with --disable option!"
-    if commandExtraArgs cmd < 0
-      then runWithHooks specops extra
-      else if length extra > commandExtraArgs cmd
-           then fail $ "Bad argument: `"++unwords extra++"'\n"++
-                       getCommandMiniHelp msuper cmd
-           else if length extra < commandExtraArgs cmd
-                then fail $ "Missing argument:  " ++
-                            nth_arg (length extra + 1) ++
-                            "\n" ++ getCommandMiniHelp msuper cmd
-                else runWithHooks specops extra
+    case extraArgumentsError extra cmd msuper of
+        Nothing     -> runWithHooks specops extra
+        Just msg    -> fail msg
        where nubopts = nubOptions (uncurry (++) $ commandAlloptions cmd)
-             nth_arg n = nth_of n (commandExtraArgHelp cmd)
-             nth_of 1 (h:_) = h
-             nth_of n (_:hs) = nth_of (n-1) hs
-             nth_of _ [] = "UNDOCUMENTED"
              runWithHooks os ex = do
                here <- getCurrentDirectory
                checkMatchSyntax os
@@ -131,10 +120,8 @@ considerRunning msuper cmd opts old_extra = do
                when (Timings `elem` os) setTimingsMode
                when (Debug `elem` os) setDebugMode
                when (DebugHTTP `elem` os) setDebugHTTP
-               when (Verbose `elem` os) setVerboseMode
                when (Quiet `elem` os) $ setProgressMode False
                when (NoHTTPPipelining `elem` os) $ disableHTTPPipelining
-               unless (SSHControlMaster `elem` os) setSshControlMasterDisabled
                unless (Quiet `elem` os) $ atexit $ doCRCWarnings (Verbose `elem` os)
                -- actually run the command and its hooks
                preHookExitCode <- runPrehook os here
@@ -146,6 +133,31 @@ considerRunning msuper cmd opts old_extra = do
                           postHookExitCode <- runPosthook os here
                           exitWith postHookExitCode
 
+-- Checks if the number of extra arguments matches the number of extra
+-- arguments supported by the command as specified in `commandExtraArgs`.
+-- Extra arguments are arguments that follow the command but aren't
+-- considered a flag. In `darcs push xyz`, xyz would be an extra argument.
+extraArgumentsError :: [String]             -- extra commands provided by user
+                    -> DarcsCommand
+                    -> Maybe DarcsCommand
+                    -> Maybe String
+extraArgumentsError extra cmd msuper
+    | extraArgsCmd < 0 = Nothing
+    | extraArgsInput > extraArgsCmd = Just badArg
+    | extraArgsInput < extraArgsCmd = Just missingArg
+    | otherwise = Nothing
+        where
+            extraArgsInput = length extra
+            extraArgsCmd = commandExtraArgs cmd
+            badArg     = "Bad argument: `" ++ unwords extra ++
+                         "'\n" ++ getCommandMiniHelp msuper cmd
+            missingArg = "Missing argument:  " ++ nthArg (length extra + 1) ++
+                         "\n" ++ getCommandMiniHelp msuper cmd
+            nthArg n       = nthOf n (commandExtraArgHelp cmd)
+            nthOf 1 (h:_)  = h
+            nthOf n (_:hs) = nthOf (n-1) hs
+            nthOf _ []     = "UNDOCUMENTED"
+
 addCommandDefaults :: DarcsCommand -> [DarcsFlag] -> IO [DarcsFlag]
 addCommandDefaults cmd already = do
   let (opts1, opts2) = commandAlloptions cmd
@@ -153,7 +165,7 @@ addCommandDefaults cmd already = do
   return $ already ++ defaults
 
 getOptionsOptions :: [OptDescr DarcsFlag] -> String
-getOptionsOptions = concat . intersperse "\n" . concatMap goo
+getOptionsOptions = intercalate "\n" . concatMap goo
  where
   goo (Option _ os _ _) = map ("--"++) os
 
@@ -177,4 +189,4 @@ runRawSupercommand super args = do
             then fail $ "Command " ++ (commandName super) ++
                       " disabled with --disable option!"
             else fail $ "Invalid subcommand!\n\n" ++ subusage super
-    (_,_,ermsgs) -> do fail $ chompNewline(unlines ermsgs)
+    (_,_,ermsgs) -> fail $ chompNewline(unlines ermsgs)
