@@ -15,9 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# OPTIONS_GHC -fglasgow-exts #-}
 {-# LANGUAGE CPP, ForeignFunctionInterface #-}
--- , ScopedTypeVariables, TypeOperators, PatternGuards #-}
 
 #include "gadts.h"
 
@@ -25,9 +23,8 @@ module Darcs.SelectChanges (selectChanges, WhichChanges(..), viewChanges, withSe
                             filterOutConflicts, runSelection, selectionContextPrim,
                             selectionContext
                      ) where
-import System.IO
 import Data.List ( intersperse )
-import Data.Maybe ( catMaybes, isJust, fromJust )
+import Data.Maybe ( isJust, isNothing )
 import Data.Char ( toUpper )
 import Control.Monad ( when, (>=>) )
 import Control.Monad.Identity ( Identity(..) )
@@ -38,20 +35,21 @@ import System.Exit ( exitWith, ExitCode(ExitSuccess) )
 
 import English ( Noun(..), englishNum  )
 import Darcs.Arguments ( showFriendly )
-import Darcs.Hopefully ( PatchInfoAnd, hopefully, n2pia )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, hopefully, n2pia )
 import Darcs.Repository ( Repository, readRepo, unrecordedChanges )
-import Darcs.Patch ( RepoPatch, Patchy, Prim, summary,
+import Darcs.Patch ( RepoPatch, Patchy, PrimPatch, summary,
                      invert, listTouchedFiles,
                      commuteFLorComplain, fromPrims, anonymous )
 import Darcs.Patch.Set ( newset2RL )
+import Darcs.Patch.Apply( ApplyState )
 import qualified Darcs.Patch ( thing, things )
 import Darcs.Patch.Split ( Splitter(..) )
 import Darcs.Witnesses.Ordered ( FL(..), RL(..), (:>)(..), (:||:)(..),
-                       (+>+), lengthFL, concatRL, mapFL_FL,
+                       (+>+), lengthFL, mapFL_FL,
                        spanFL, spanFL_M, reverseFL, (+<+), mapFL, filterFL )
 import Darcs.Witnesses.WZipper( FZipper(..), left, right
-                              , rightmost, lengthFZ, focus
-                              , toEnd)
+                              , rightmost, nullFZ
+                              , toEnd, toStart)
 import Darcs.Patch.Choices ( PatchChoices, patchChoices,
                              patchChoicesTpsSub,
                              forceFirst, forceLast, makeUncertain, tag,
@@ -69,11 +67,14 @@ import Darcs.Patch.Permutations ( partitionConflictingFL, selfCommuter, commuter
 import qualified Darcs.Patch.TouchesFiles as TouchesFiles
 import Darcs.PrintPatch ( printFriendly, printPatch, printPatchPager )
 import Darcs.Match ( haveNonrangeMatch, matchAPatch, matchAPatchread )
-import Darcs.Flags ( DarcsFlag( Summary, DontGrabDeps, Verbose, DontPromptForDependencies, SkipConflicts), isInteractive )
+import Darcs.Flags ( DarcsFlag( Summary, DontGrabDeps, Verbose, DontPromptForDependencies
+                              , SkipConflicts )
+                   , isInteractive, UseIndex(..), ScanKnown(..) )
 import Darcs.Witnesses.Sealed ( FlippedSeal(..), flipSeal, seal2, unseal2, Sealed(..), Sealed2(..) )
 import Darcs.Utils ( askUser, promptChar, PromptConfig(..) )
 import Darcs.Lock ( editText )
 import Printer ( prefix, putDocLn )
+import Storage.Hashed.Tree( Tree )
 
 -- | When asking about patches, we either ask about them in
 -- oldest-first or newest first (with respect to the current ordering
@@ -109,29 +110,32 @@ type MatchCriterion p = WhichChanges -> [DarcsFlag] -> Sealed2 p -> Bool
 -- patches. See "PatchSelectionM"
 data PatchSelectionContext p = PSC { opts :: [DarcsFlag]
                                    , splitter :: Maybe (Splitter p)
-                                   , files :: [FilePath]
+                                   , files :: Maybe [FilePath]
                                    , matchCriterion :: MatchCriterion p
-                                   , jobname :: String }
+                                   , jobname :: String
+                                   , pristine :: Maybe (Tree IO)}
 
 -- | A 'PatchSelectionContext' for selecting 'Prim' patches.
-selectionContextPrim ::  String -> [DarcsFlag] -> Maybe (Splitter Prim)
-                 -> [FilePath] -> PatchSelectionContext Prim
-selectionContextPrim jn o spl fs =
+selectionContextPrim ::  PrimPatch prim => String -> [DarcsFlag] -> Maybe (Splitter prim)
+                 -> Maybe [FilePath] -> Maybe (Tree IO) -> PatchSelectionContext prim
+selectionContextPrim jn o spl fs p =
  PSC { opts = o
      , splitter = spl
      , files = fs
      , matchCriterion = triv
-     , jobname = jn }
+     , jobname = jn
+     , pristine = p }
 
 -- | A 'PatchSelectionContext' for selecting full patches ('PatchInfoAnd' patches)
 selectionContext :: (RepoPatch p) => String -> [DarcsFlag] -> Maybe (Splitter (PatchInfoAnd p))
-                 -> [FilePath] -> PatchSelectionContext (PatchInfoAnd p)
+                 -> Maybe [FilePath] -> PatchSelectionContext (PatchInfoAnd p)
 selectionContext jn o spl fs =
  PSC { opts = o
      , splitter = spl
      , files = fs
      , matchCriterion = iswanted
-     , jobname = jn }
+     , jobname = jn
+     , pristine = Nothing }
 
 -- | The dynamic parameters for interactive selection of patches.
 data InteractiveSelectionContext p C(x y) = ISC { total :: Int
@@ -163,12 +167,12 @@ triv = \ _ _ _ -> True
 -- opts'
 iswanted :: Patchy p => MatchCriterion (PatchInfoAnd p)
 iswanted whch opts' =
-    unseal2 (iw whch opts')
+    unseal2 (iw whch)
         where
-          iw First o = matchAPatch o . hopefully
-          iw Last o = matchAPatch o . hopefully
-          iw LastReversed o = matchAPatch o . hopefully . invert
-          iw FirstReversed o = matchAPatch o . hopefully . invert
+          iw First = matchAPatch opts' . hopefully
+          iw Last = matchAPatch opts' . hopefully
+          iw LastReversed = matchAPatch opts' . hopefully . invert
+          iw FirstReversed = matchAPatch opts' . hopefully . invert
 
 liftR :: Monad m => Reader r a -> ReaderT r m a
 liftR = asks . runReader
@@ -179,7 +183,7 @@ runSelection :: (Patchy p) => PatchSelection p C(x y) -> PatchSelectionContext p
 runSelection = runReaderT
 
 -- | Select patches from a @FL@.
-selectChanges :: forall p C(x y) . Patchy p =>
+selectChanges :: forall p C(x y) . (Patchy p, ApplyState p ~ Tree) =>
                 WhichChanges -> FL p C(x y)
                              -> PatchSelection p C(x y)
 selectChanges First = sc1 First
@@ -192,7 +196,7 @@ selectChanges LastReversed = return . invert
                              >=> sc1 LastReversed
                              >=> return . invertC
 
-sc1 :: forall p C(x y) . Patchy p =>
+sc1 :: forall p C(x y) . (Patchy p, ApplyState p ~ Tree) =>
                 WhichChanges -> FL p C(x y)
                              -> PatchSelection p C(x y)
 sc1 whch =
@@ -214,8 +218,8 @@ repr Last (Sealed2 p) = Sealed2 p
 repr FirstReversed (Sealed2 p) = Sealed2 (invert p)
 
 -- | The equivalent of 'selectChanges' for the @darcs changes@ command
-viewChanges :: Patchy p => [DarcsFlag] -> [Sealed2 p] -> IO ()
-viewChanges opts ps = textView opts Nothing 0 [] ps
+viewChanges :: (Patchy p, ApplyState p ~ Tree) => [DarcsFlag] -> [Sealed2 p] -> IO ()
+viewChanges opts' ps = textView opts' Nothing 0 [] ps
 
 -- | The type of the answers to a "shall I [wiggle] that [foo]?" question
 -- They are found in a [[KeyPress]] bunch, each list representing a set of
@@ -241,71 +245,86 @@ keysFor :: [[KeyPress]] -> [Char]
 keysFor = concatMap (map kp)
 
 -- | The function for selecting a patch to amend record. Read at your own risks.
-withSelectedPatchFromRepo :: forall p C(r u t). RepoPatch p => String -> Repository p C(r u t) -> [DarcsFlag]
-                              -> (FORALL(a) (FL (PatchInfoAnd p) :> PatchInfoAnd p) C(a r) -> IO ()) -> IO ()
+withSelectedPatchFromRepo ::
+       forall p C(r u t). (RepoPatch p, ApplyState p ~ Tree)
+    => String   -- name of calling command (always "amend" as of now)
+    -> Repository p C(r u t)
+    -> [DarcsFlag]
+    -> (FORALL(a) (FL (PatchInfoAnd p) :> PatchInfoAnd p) C(a r) -> IO ())
+    -> IO ()
 withSelectedPatchFromRepo jn repository o job = do
-    p_s <- readRepo repository
-    sp <- wspfr jn (matchAPatchread o)
-                              (newset2RL p_s) NilFL
+    patchSet <- readRepo repository
+    sp <- wspfr jn (matchAPatchread o) (newset2RL patchSet) NilFL
     case sp of
-     Just (FlippedSeal (skipped :> selected)) -> job (skipped :> selected)
-     Nothing -> do putStrLn $ "Cancelling "++jn++" since no patch was selected."
+        Just (FlippedSeal (skipped :> selected')) -> job (skipped :> selected')
+        Nothing ->
+            putStrLn $ "Cancelling " ++ jn ++ " since no patch was selected."
 
--- | This ensures that the selected patch commutes freely with the skipped patches, including pending
--- and also that the skipped sequences has an ending context that matches the recorded state, z,
--- of the repository.
-wspfr :: RepoPatch p => String -> (FORALL(a b) (PatchInfoAnd p) C(a b) -> Bool)
-      -> RL (PatchInfoAnd p) C(x y) -> FL (PatchInfoAnd p) C(y u)
+-- | This ensures that the selected patch commutes freely with the skipped
+-- patches, including pending and also that the skipped sequences has an
+-- ending context that matches the recorded state, z, of the repository.
+wspfr :: (RepoPatch p, ApplyState p ~ Tree)
+      => String
+      -> (FORALL(a b) (PatchInfoAnd p) C(a b) -> Bool)
+      -> RL (PatchInfoAnd p) C(x y)
+      -> FL (PatchInfoAnd p) C(y u)
       -> IO (Maybe (FlippedSeal (FL (PatchInfoAnd p) :> (PatchInfoAnd p)) C(u)))
 wspfr _ _ NilRL _ = return Nothing
-wspfr jn matches (p:<:pps) skipped
+wspfr jn matches remaining@(p:<:pps) skipped
     | not $ matches p = wspfr jn matches pps (p:>:skipped)
     | otherwise =
     case commuteFLorComplain (p :> skipped) of
     Left _  -> do putStrLn "\nSkipping depended-upon patch:"
-                  printFriendly [] p
+                  printFriendly Nothing [] p
                   wspfr jn matches pps (p:>:skipped)
     Right (skipped' :> p') -> do
-      printFriendly [] p
-      let repeat_this  = wspfr jn matches (p:<:pps) skipped
-          basic_options =
-                    [[ KeyPress 'y' (jn++" this patch")
-                     , KeyPress 'n' ("don't "++jn++" it")
+        printFriendly Nothing [] p
+        yorn <- promptChar $
+                    PromptConfig { pPrompt = prompt'
+                                 , pBasicCharacters = keysFor basicOptions
+                                 , pAdvancedCharacters = keysFor advancedOptions
+                                 , pDefault = Just 'n'
+                                 , pHelp = "?h" }
+        case yorn of
+            'y' -> return $ Just $ flipSeal $ skipped' :> p'
+            'n' -> nextPatch
+            'j' -> nextPatch
+            'k' -> case skipped of
+                     NilFL -> repeatThis
+                     (prev :>: skipped') -> wspfr jn matches (prev :<: remaining) skipped'
+            'v' -> printPatch p >> repeatThis
+            'p' -> printPatchPager p >> repeatThis
+            'x' -> do putDocLn $ prefix "    " $ summary p
+                      repeatThis
+            'q' -> do putStrLn $ jnCapital ++ " cancelled."
+                      exitWith $ ExitSuccess
+            _   -> do putStrLn $ helpFor jn basicOptions advancedOptions
+                      repeatThis
+  where jnCapital = (toUpper $ head jn) : tail jn
+        repeatThis = wspfr jn matches (p:<:pps) skipped
+        prompt' = "Shall I " ++ jn ++ " this patch?"
+        nextPatch = wspfr jn matches pps (p:>:skipped)
+        basicOptions =
+                    [[ KeyPress 'y' (jn ++ " this patch")
+                     , KeyPress 'n' ("don't " ++ jn ++ " it")
+                     , KeyPress 'j' "skip to next patch"
+                     , KeyPress 'k' "back up to previous patch"
                     ]]
-          advanced_options =
+        advancedOptions =
                     [[ KeyPress 'v' "view this patch in full"
                      , KeyPress 'p' "view this patch in full with pager"
                      , KeyPress 'x' "view a summary of this patch"
-                     , KeyPress 'q' ("cancel "++jn)
+                     , KeyPress 'q' ("cancel " ++ jn)
                     ]]
-      let prompt  = "Shall I "++jn++" this patch?"
-      yorn <- promptChar $ PromptConfig { pPrompt = prompt
-                                        , pBasicCharacters = keysFor basic_options
-                                        , pAdvancedCharacters = keysFor advanced_options
-                                        , pDefault = Just 'n'
-                                        , pHelp = "?h" }
-      case yorn of
-        'y' -> return $ Just $ flipSeal $ skipped' :> p'
-        'n' -> wspfr jn matches pps (p:>:skipped)
-        'v' -> printPatch p >> repeat_this
-        'p' -> printPatchPager p >> repeat_this
-        'x' -> do putDocLn $ prefix "    " $ summary p
-                  repeat_this
-        'q' -> do putStrLn $ jn_cap++" cancelled."
-                  exitWith $ ExitSuccess
-        _   -> do putStrLn $ helpFor jn basic_options advanced_options
-                  repeat_this
-  where jn_cap = (toUpper $ head jn) : tail jn
 
 -- After selecting with a splitter, the results may not be canonical
 canonizeAfterSplitter :: (FL p :> FL p) C(x y) -> Reader (PatchSelectionContext p) ((FL p :> FL p) C(x y))
 canonizeAfterSplitter (x :> y) =
-    do spl' <- asks splitter
-       case spl' of
-         Nothing -> return (x:>y)
-         Just spl -> return $ canonizeSplit spl x :> canonizeSplit spl y
+    do mspl <- asks splitter
+       let canonizeIfNeeded = maybe id canonizeSplit mspl
+       return $ canonizeIfNeeded x :> canonizeIfNeeded y
 
-realSelectChanges :: forall p C(x y). Patchy p
+realSelectChanges :: forall p C(x y). (Patchy p, ApplyState p ~ Tree)
                                 => WhichChanges
                                 -> PatchChoices p C(x y)
                                 -> PatchSelectionM p IO (PatchChoices p C(x y))
@@ -328,7 +347,7 @@ deselectUnwanted :: forall p C(x y) . Patchy p => WhichChanges ->
 deselectUnwanted whichch pc =
     do
       o <- asks opts
-      c <- (asks matchCriterion)
+      c <- asks matchCriterion
       let iswanted_ = c whichch o . seal2 . tpPatch
           select = if forward
                    then forceMatchingFirst iswanted_
@@ -336,12 +355,12 @@ deselectUnwanted whichch pc =
           deselect = if forward
                      then forceMatchingLast (not . iswanted_)
                      else forceMatchingFirst (not . iswanted_)
-      if haveNonrangeMatch o
-       then if DontGrabDeps `elem` o
-            then return $ deselect pc
-            else do
-                 return . demote $ select pc
-       else return pc
+      return $
+        if haveNonrangeMatch o
+            then if DontGrabDeps `elem` o
+                 then deselect pc
+                 else demote $ select pc
+            else pc
     where
       forward = not $ backward whichch
       demote = if forward
@@ -350,7 +369,7 @@ deselectUnwanted whichch pc =
 
 -- | Selects the patches matching the match criterion, and puts them first or last
 -- according to whch, while respecting any dependencies.
-patchesToConsider :: Patchy p
+patchesToConsider :: (Patchy p, ApplyState p ~ Tree)
                      => WhichChanges
                      -> FL p C(x y)
                      -> Reader (PatchSelectionContext p) (PatchChoices p C(x y))
@@ -365,7 +384,7 @@ patchesToConsider whch ps =
                 FirstReversed -> TouchesFiles.selectNotTouching
                 LastReversed -> TouchesFiles.deselectNotTouching
           everything = patchChoices ps
-      if null fs && not (haveNonrangeMatch o)
+      if isNothing fs && not (haveNonrangeMatch o)
          then return everything
          else do notUnwanted <- deselectUnwanted whch everything
                  return $ deselectNotTouching fs notUnwanted
@@ -395,8 +414,8 @@ liftChoices :: forall p a C(x y) . Patchy p =>
                    -> InteractiveSelectionM p C(x y) a
 liftChoices act = do
   ch <- gets choices
-  let (result, ch') = runIdentity $ runStateT act ch
-  modify $ \isc -> isc {choices = ch}
+  let (result, _) = runIdentity $ runStateT act ch
+  modify $ \isc -> isc {choices = ch} -- Should this be ch or the result of runState?
   return result
 
 -- | @justDone n@ notes that @n@ patches have just been processed
@@ -404,20 +423,25 @@ justDone :: Patchy p => Int -> InteractiveSelectionM p C(x y) ()
 justDone n = modify $ \isc -> isc{ current = current isc + n}
 
 -- | The actual interactive selection process.
-textSelect :: forall p C(x y) . Patchy p => WhichChanges ->
+textSelect :: forall p C(x y) . (Patchy p, ApplyState p ~ Tree) => WhichChanges ->
              FL (TaggedPatch p) C(x y) -> PatchChoices p C(x y)
              -> PatchSelectionM p IO (PatchChoices p C(x y))
-textSelect whch tps pcs = do
+textSelect whch tps' pcs = do
     userSelection <- execStateT (skipMundane whch >>
                                  showCur whch >>
-                                 textSelect' whch) $
-                     ISC { total = lengthFL tps
+                                 textSelectIfAny) $
+                     ISC { total = lengthFL tps'
                          , current = 0
-                         , tps = FZipper NilRL tps
+                         , tps = FZipper NilRL tps'
                          , choices = pcs }
     return $ choices userSelection
+    where textSelectIfAny = do
+            z <- gets tps
+            if rightmost z
+              then return ()
+              else textSelect' whch
 
-textSelect' :: Patchy p => WhichChanges ->
+textSelect' :: (Patchy p, ApplyState p ~ Tree) => WhichChanges ->
               InteractiveSelectionM p C(x y) ()
 textSelect' whch = do
   z <- gets tps
@@ -455,8 +479,9 @@ optionsQuit jn someThings =
 
 optionsNav :: String -> [KeyPress]
 optionsNav aThing =
-    [ KeyPress 'j' ("skip to next "++ aThing)
-    , KeyPress 'k' ("back up to previous "++ aThing) ]
+  [ KeyPress 'j' ("skip to next "++ aThing)
+  ,  KeyPress 'k' ("back up to previous "++ aThing)
+  , KeyPress 'g' ("start over from the first "++aThing)]
 
 optionsSplit :: Maybe (Splitter a) -> String -> [KeyPress]
 optionsSplit split aThing
@@ -620,6 +645,12 @@ skipAll :: forall p C(x y) . Patchy p =>
 skipAll = do
   modify $ \isc -> isc {tps = toEnd $ tps isc}
 
+backAll :: forall p C(x y) . Patchy p =>
+          InteractiveSelectionM p C(x y) ()
+backAll = do
+  modify $ \isc -> isc {tps = toStart $ tps isc
+                       ,current = 0}
+
 isSingleFile :: Patchy p => p C(x y) -> Bool
 isSingleFile p = length (listTouchedFiles p) == 1
 
@@ -673,7 +704,7 @@ promptUser single def = do
                                      }
 
 -- | Ask the user what to do with the next patch.
-textSelectOne :: forall p C(x y). Patchy p => WhichChanges
+textSelectOne :: forall p C(x y). (Patchy p, ApplyState p ~ Tree) => WhichChanges
             -> InteractiveSelectionM p C(x y) ()
 textSelectOne whichch = do
  c <- currentPatch
@@ -683,14 +714,13 @@ textSelectOne whichch = do
        do
          jn <- asks jobname
          spl <- asks splitter
-         o <- asks opts
          let singleFile = isSingleFile (tpPatch tp)
              reprCur = repr whichch (Sealed2 (tpPatch tp))
          (basicOptions,advancedOptions) <- options singleFile
          theSlot <- liftChoices $ patchSlot' tp
          let
              the_default = getDefault (whichch == Last || whichch == FirstReversed) theSlot
-             jn_cap = (toUpper $ head jn) : tail jn
+             jnCapital = (toUpper $ head jn) : tail jn
          yorn <- promptUser singleFile the_default
          let nextPatch = skipMundane whichch >> showCur whichch
          case yorn of
@@ -709,43 +739,46 @@ textSelectOne whichch = do
                'l' -> printSelected whichch >> showCur whichch
                'x' -> liftIO $ unseal2 printSummary reprCur
                'd' -> skipAll
+               'g' -> backAll >> showCur whichch
                'a' ->
                    do
                      askConfirmation
                      modChoices $ selectAllMiddles (whichch == Last || whichch == FirstReversed)
                      skipAll
                'q' -> liftIO $
-                      do putStrLn $ jn_cap++" cancelled."
+                      do putStrLn $ jnCapital++" cancelled."
                          exitWith $ ExitSuccess
                'j' -> skipOne >> showCur whichch
                'k' -> backOne >> showCur whichch
-               _   -> do liftIO . putStrLn $ helpFor jn basicOptions advancedOptions
+               _   -> do
+                 liftIO . putStrLn $ helpFor jn basicOptions advancedOptions
 
 -- | Shows the current patch as it should be seen by the user.
-showCur :: forall p C(x y) . Patchy p => WhichChanges
+showCur :: forall p C(x y) . (Patchy p, ApplyState p ~ Tree) => WhichChanges
            -> InteractiveSelectionM p C(x y) ()
 showCur whichch = do
   o <- asks opts
+  p <- asks pristine
   c <- currentPatch
   case c of
       Nothing -> return ()
       Just (Sealed2 tp) -> do
              let reprCur = repr whichch (Sealed2 (tpPatch tp))
-             liftIO . (unseal2 (printFriendly o)) $ reprCur
+             liftIO . (unseal2 (printFriendly p o)) $ reprCur
 
 -- | The interactive part of @darcs changes@
-textView :: forall p C(x y u r s). Patchy p => [DarcsFlag] -> Maybe Int -> Int
+textView :: forall p . (Patchy p, ApplyState p ~ Tree) => [DarcsFlag] -> Maybe Int -> Int
             -> [Sealed2 p] -> [Sealed2 p]
             -> IO ()
 textView _ _ _ _ [] = return ()
 textView o n_max n
             ps_done ps_todo@(p:ps_todo') = do
-      unseal2 (printFriendly o) p
-      repeat_this -- prompt the user
+      unseal2 (printFriendly Nothing o) p
+      repeatThis -- prompt the user
     where
         prev_patch :: IO ()
         prev_patch = case ps_done of
-                       [] -> repeat_this
+                       [] -> repeatThis
                        (p':ps_done') ->
                          textView o
                             n_max (n-1) ps_done' (p':ps_todo)
@@ -757,43 +790,46 @@ textView o n_max n
                                    n ps_done []
                          _ -> textView o n_max
                                   (n+1) (p:ps_done) ps_todo'
+        first_patch = textView o n_max 0 [] (ps_done++ps_todo)
         options_yn =
           [ KeyPress 'y' "view this patch and go to the next"
           , KeyPress 'n' "skip to the next patch" ]
-        optionsView =
+        optionsView' =
           [ KeyPress 'v' "view this patch in full"
           , KeyPress 'p' "view this patch in full with pager" ]
-        optionsSummary =
+        optionsSummary' =
           [ KeyPress 'x' "view a summary of this patch" ]
-        optionsNav =
+        optionsNav' =
           [ KeyPress 'q' "quit view changes"
           , KeyPress 'k' "back up to previous patch"
           , KeyPress 'j' "skip to next patch"
+          , KeyPress 'g' "start over from the first patch"
           , KeyPress 'c' "count total patch number" ]
         basicOptions = [ options_yn ]
         advancedOptions =
-                     [ optionsView ++
-                       if Summary `elem` o then [] else optionsSummary ]
-                  ++ [ optionsNav ]
-        prompt = "Shall I view this patch? "
+                     [ optionsView' ++
+                       if Summary `elem` o then [] else optionsSummary' ]
+                  ++ [ optionsNav' ]
+        prompt' = "Shall I view this patch? "
                ++ "(" ++ show (n+1) ++ "/" ++ maybe "?" show n_max ++ ")"
-        repeat_this :: IO ()
-        repeat_this = do
-          yorn <- promptChar (PromptConfig prompt (keysFor basicOptions) (keysFor advancedOptions) (Just 'n') "?h")
+        repeatThis :: IO ()
+        repeatThis = do
+          yorn <- promptChar (PromptConfig prompt' (keysFor basicOptions) (keysFor advancedOptions) (Just 'n') "?h")
           case yorn of
             'y' -> unseal2 printPatch p >> next_patch
             'n' -> next_patch
-            'v' -> unseal2 printPatch p >> repeat_this
-            'p' -> unseal2 printPatchPager p >> repeat_this
+            'v' -> unseal2 printPatch p >> repeatThis
+            'p' -> unseal2 printPatchPager p >> repeatThis
             'x' -> do putDocLn $ prefix "    " $ unseal2 summary p
-                      repeat_this
+                      repeatThis
             'q' -> exitWith ExitSuccess
             'k' -> prev_patch
             'j' -> next_patch
+            'g' -> first_patch
             'c' -> textView o
                        count_n_max n ps_done ps_todo
             _   -> do putStrLn $ helpFor "view changes" basicOptions advancedOptions
-                      repeat_this
+                      repeatThis
         count_n_max | isJust n_max = n_max
                     | otherwise    = Just $ length ps_done + length ps_todo
 
@@ -846,7 +882,7 @@ getDefault False InLast  = 'n'
 
 -- |Optionally remove any patches (+dependencies) from a sequence that
 -- conflict with the recorded or unrecorded changes in a repo
-filterOutConflicts :: RepoPatch p
+filterOutConflicts :: (RepoPatch p, ApplyState p ~ Tree)
                    => [DarcsFlag]                                    -- ^Command-line options. Only 'SkipConflicts' is
                                                                      -- significant; filtering will happen iff it is present
                    -> RL (PatchInfoAnd p) C(x t)                     -- ^Recorded patches from repository, starting from
@@ -857,7 +893,8 @@ filterOutConflicts :: RepoPatch p
 filterOutConflicts o us repository them
   | SkipConflicts `elem` o
      = do let commuter = commuterIdRL selfCommuter
-          unrec <- fmap n2pia . (anonymous . fromPrims) =<< unrecordedChanges [] repository []
+          unrec <- fmap n2pia . (anonymous . fromPrims)
+                     =<< unrecordedChanges (UseIndex, ScanKnown) repository Nothing
           them' :> rest <- return $ partitionConflictingFL commuter them (unrec :<: us)
           return (check rest, Sealed them')
   | otherwise
