@@ -2,6 +2,9 @@
 #if __GLASGOW_HASKELL__ >= 700
 {-# LANGUAGE ImpredicativeTypes #-}
 #endif
+#if __GLASGOW_HASKELL__ >= 708
+{-# LANGUAGE AllowAmbiguousTypes #-}
+#endif
 --  Copyright (C) 2002-2005,2007 David Roundy
 --
 --  This program is free software; you can redistribute it and/or modify
