@@ -28,7 +28,7 @@ import Data.Maybe ( isNothing )
 import Data.Text as T ( find, any )
 import Data.Text.Encoding ( decodeUtf8With )
 import Data.Text.Encoding.Error ( lenientDecode )
-import Foreign ( unsafePerformIO )
+import System.IO.Unsafe ( unsafePerformIO )
 import Test.QuickCheck ( Arbitrary(arbitrary), oneof, listOf, choose, shrink
                        , Gen )
 import Test.Framework.Providers.QuickCheck2 ( testProperty )
