@@ -18,21 +18,21 @@
 {-# LANGUAGE CPP #-}
 module Darcs.TheCommands ( commandControlList ) where
 
-import Prelude hiding ( log )
+import Prelude ()
 import Darcs.Commands.Add ( add )
-import Darcs.Commands.AmendRecord ( amendrecord )
+import Darcs.Commands.AmendRecord ( amendrecord, amendunrecord )
 import Darcs.Commands.Annotate ( annotate )
 import Darcs.Commands.Apply ( apply )
 import Darcs.Commands.Changes ( changes, log )
-import Darcs.Commands.Check ( check )
+import Darcs.Commands.Check ( check, repair )
 import Darcs.Commands.Convert ( convert )
-import Darcs.Commands.Diff
+import Darcs.Commands.Diff ( diffCommand )
 import Darcs.Commands.Dist ( dist )
 import Darcs.Commands.Get ( get, clone )
 import Darcs.Commands.GZCRCs ( gzcrcs )
 import Darcs.Commands.Init ( initialize )
 import Darcs.Commands.Show ( showCommand, list, query )
-import Darcs.Commands.MarkConflicts ( markconflicts, resolve )
+import Darcs.Commands.MarkConflicts ( markconflicts )
 import Darcs.Commands.Move ( move, mv )
 import Darcs.Commands.Optimize ( optimize )
 import Darcs.Commands.Pull ( pull, fetch )
@@ -40,18 +40,18 @@ import Darcs.Commands.Push ( push )
 import Darcs.Commands.Put ( put )
 import Darcs.Commands.Record ( record, commit )
 import Darcs.Commands.Remove ( remove, rm, unadd )
-import Darcs.Commands.Repair ( repair )
 import Darcs.Commands.Replace ( replace )
 import Darcs.Commands.Revert ( revert )
 import Darcs.Commands.Rollback ( rollback )
 import Darcs.Commands.Send ( send )
 import Darcs.Commands.SetPref ( setpref )
 import Darcs.Commands.Tag ( tag )
+import Darcs.Commands.Test ( test )
 import Darcs.Commands.TrackDown ( trackdown )
 import Darcs.Commands.TransferMode ( transferMode )
 import Darcs.Commands.Unrecord ( unrecord, unpull, obliterate )
 import Darcs.Commands.Unrevert ( unrevert )
-import Darcs.Commands.WhatsNew ( whatsnew )
+import Darcs.Commands.WhatsNew ( whatsnew, status )
 import Darcs.Commands ( CommandControl(CommandData,HiddenCommand,GroupName) )
 
 -- | The commands that darcs knows about (e.g. whatsnew, record),
@@ -65,12 +65,13 @@ commandControlList = [GroupName "Changing and querying the working copy:",
                 CommandData replace,
                 CommandData revert,
                 CommandData unrevert,
-                CommandData whatsnew,
+                CommandData whatsnew, HiddenCommand status,
                 GroupName "Copying changes between the working copy and the repository:",
                 CommandData record, HiddenCommand commit,
                 CommandData unrecord,
                 CommandData amendrecord,
-                CommandData markconflicts, HiddenCommand resolve,
+                HiddenCommand amendunrecord,
+                CommandData markconflicts,
                 GroupName "Direct modification of the repository:",
                 CommandData tag,
                 CommandData setpref,
@@ -79,6 +80,7 @@ commandControlList = [GroupName "Changing and querying the working copy:",
                 CommandData changes, HiddenCommand log,
                 CommandData annotate,
                 CommandData dist,
+                CommandData test,
                 CommandData trackdown,
                 CommandData showCommand, HiddenCommand list, HiddenCommand query,
                 HiddenCommand transferMode,
