@@ -31,7 +31,7 @@ import Prelude hiding ( catch )
 
 import qualified System.Directory ( renameFile, getCurrentDirectory, removeFile )
 import Control.Exception ( catch, IOException )
-import qualified Control.Exception ( block )
+import qualified Control.Exception ( mask )
 import qualified System.IO.Error ( isDoesNotExistError, ioError )
 
 #else
@@ -96,7 +96,7 @@ sigALRM = 0
 renameFile :: FilePath
            -> FilePath
            -> IO ()
-renameFile old new = Control.Exception.block $
+renameFile old new = Control.Exception.mask $ \_ ->
    System.Directory.renameFile old new
    `catch` \(_ :: IOException) ->
    do System.Directory.removeFile new
