@@ -4,10 +4,6 @@
 
 . lib
 
-rm -rf temp1
-
-mkdir temp1
-cd temp1
 darcs init
 date > foo
 darcs add foo
@@ -32,9 +28,7 @@ grep 'mv foo to d' log
 echo 'That has such people in it' > directory/foo
 darcs add directory/foo
 darcs record -m 'mv foo then add new foo' -a
-darcs annotate directory/bar > log
-grep 'How beauteous mankind is' log
+darcs annotate directory/bar | tee log
 grep 'O brave new world' log
-cd ..
-
-rm -rf temp1
+grep "mv foo then add new foo" log
+not grep "unknown" log
