@@ -57,4 +57,6 @@ cd ..
 darcs get ac abc2
 cd abc2
 darcs pull -a ../bc
+darcs changes
 
+test `darcs changes | fgrep -c '* C'` -eq 1
