@@ -15,4 +15,5 @@ abs_to_context="${DIR}/my_context"
 cd ..
 rm -rf temp2
 darcs get temp1 --context="${abs_to_context}" temp2
-rm -rf temp1 temp2
+darcs changes --context --repo temp2 > repo2_context
+diff -u "${abs_to_context}" repo2_context
