@@ -1,12 +1,10 @@
-#!/bin/sh
+#!/usr/bin/env bash
 set -ev
 
-test $DARCS || DARCS=$PWD/../darcs
-
 rm -rf tempOld tempA tempB
 mkdir tempOld tempA
 cd tempOld
-$DARCS initialize
+darcs initialize
 echo record author me > _darcs/prefs/defaults
 echo ALL all >> _darcs/prefs/defaults
 #echo ALL verbose >> _darcs/prefs/defaults
@@ -19,33 +17,33 @@ echo E >> foo
 echo F >> foo
 echo G >> foo
 echo H >> foo
-$DARCS add foo
-$DARCS record -m Old
+darcs add foo
+darcs record -m Old
 cd ..
 
 cd tempA
-$DARCS initialize
+darcs initialize
 cp ../tempOld/_darcs/prefs/defaults _darcs/prefs
-$DARCS pull ../tempOld
+darcs pull ../tempOld
 cp foo temp
 cat temp | grep -v A | grep -v B | grep -v D | sed s/E/e/ \
     | grep -v G | sed s/H/h/ > foo
-$DARCS record -m A
+darcs record -m AA
 cd ..
 
-$DARCS get tempOld tempB
+darcs get tempOld tempB
 cd tempB
 cp ../tempOld/_darcs/prefs/defaults _darcs/prefs
 echo 7 > foo
-$DARCS record -m B
-$DARCS pull ../tempA
-$DARCS record -m "conflict resolution"
+darcs record -m BB
+darcs pull ../tempA
+darcs record -m "conflict resolution"
 cd ..
 
 cd tempA
-$DARCS pull ../tempB
-$DARCS annotate -p B
-$DARCS annotate -p resolution
+darcs pull ../tempB
+darcs annotate -p B
+darcs annotate -p resolution
 cd ..
 
 cmp tempA/foo tempB/foo
