@@ -8,14 +8,13 @@ cd temp1
 darcs init
 cd ..
 
-# put should set default repo
+# put should not set default repo
 cd temp1
 touch 1.txt
 darcs add 1.txt
 darcs record -a -m foo 1.txt
 darcs put ../temp2
-test -e _darcs/prefs/defaultrepo
-grep temp2 _darcs/prefs/defaultrepo
+test ! -e _darcs/prefs/defaultrepo
 cd ..
 
 # put to self
