@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
-set -ev
 
-not () { "$@" && exit 1 || :; }
+. lib
 
 
 rm -rf temp1
