@@ -8,7 +8,7 @@ abort_windows
 
 rm -rf temp1
 
-export DARCS_EDITOR="/bin/cat -n"
+export DARCS_EDITOR="cat -n"
 # editor: space in command
 mkdir temp1
 cd temp1
@@ -62,7 +62,7 @@ echo y | darcs record --edit-long-comment -a -m foo file.t | grep 'END OF'
 cd ..
 rm -rf temp1
 
-export DARCS_EDITOR='/bin/echo'
+export DARCS_EDITOR='echo'
 # editor: evil filename
 mkdir temp1
 cd temp1
